#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <EzGraph.h>
#include "layout.h"
#include "draw.h"
#include "game.h"
#include "routing.h"

#include "xpm/obj/wall.xpm"
#include "xpm/obj/gagga_init.xpm"
#include "xpm/obj/block.xpm"
//#include "xpm/obj/gondin.xpm"
#include "xpm/obj/apple2.xpm"
#include "xpm/obj/chee_init.xpm"
#include "xpm/obj/trunk.xpm"

#include "xpm/Elephant_s.xpm"


rgb_t colors[] = {
    {  0,   0,   0},
    {  0,   0, 255},
    {255,   0,   0},
    {255,   0, 255},
    {  0, 255,   0},
    {  0, 255, 255},
    {255, 255,   0},
    {255, 255, 255},
};


show_route_mode_t show_route_mode = sr_none;

typedef struct {
    image_id_t iid;
    char **xpm;
} xpm_info_t;

typedef struct {
    map_element_t me;
    image_id_t iid;
} me2iid_info_t;

static xpm_info_t xpm_info_list[] =
    {
     {ii_wall, wall_xpm},
     {ii_block, block_xpm},
     //     {ii_block, gagga_init_xpm},
     {ii_food, apple_xpm},
     {ii_player, chee_init_xpm},
     {ii_trunk, trunk_xpm},
    };
#define XPM_INFO_NUM (sizeof(xpm_info_list)/sizeof(xpm_info_t))

static me2iid_info_t me2iid_info_list[] =
    {
     {me_wall, ii_wall},
     {me_block, ii_block},
     {me_food, ii_food},
     {me_player, ii_player},
     {me_player_food, ii_player},
     {me_trunk, ii_trunk},
    };
#define ME2IID_INFO_NUM \
    (sizeof(me2iid_info_list)/sizeof(me2iid_info_t))
image_id_t me2iid_table[me_end];
#define Me2Iid(me) (me2iid_table[me])

static EzImage imgs[ii_end];
static EzImage elephant_img;

void draw_init(void){
    EzSetFontSize(14);
    int i;
    elephant_img = EzReadXPM(elephant_xpm);
    for (i = 0 ; i < XPM_INFO_NUM ; i++){
        imgs[xpm_info_list[i].iid] = EzReadXPM(xpm_info_list[i].xpm);
    }
    for (i = 0 ; i < ME2IID_INFO_NUM ; i++){
        me2iid_table[me2iid_info_list[i].me] 
            = me2iid_info_list[i].iid;
    }
}

void draw_img(int x, int y, image_id_t iid){
    //        printf("draw img(%d, %d) iid:%d\n", x, y, iid);
    EzPutB(x, y,  imgs[iid]);
}

void draw_map_element(int x, int y, map_element_t me){
    draw_img(x*UNITSIZE + MAP_LTX,
             y*UNITSIZE + MAP_LTY,
             Me2Iid(me));
}

void setup_draw_screen(void){
    
}

void draw_screen(void){
    int x, y;
    map_element_t me;
    //    int player_x, player_y;
    //    EzSetColorByName("Blue");
    EzPutB(ELEPHANT_X, ELEPHANT_Y, elephant_img);

    for (y = 0; y < MAP_HEIGHT ; y++){
        for (x = 0; x < MAP_WIDTH; x++){
            me = get_map_element(x, y);
            if (me != me_blank){
                //                printf("(%d, %d) me:%d\n", x, y, me);
                if (show_route_mode != sr_value_only &&
                    show_route_mode != sr_dir_only)
                {
                    draw_map_element(x, y, me);
                }
            }
            if (show_route_mode == sr_value_overlap ||
                show_route_mode == sr_value_only)
            {
                draw_route_value(x, y);
            }
            if (show_route_mode == sr_dir_overlap ||
                show_route_mode == sr_dir_only)
            {
                draw_route_dir(x, y);
            }
            /*
            EzFillBoxB(x * UNITSIZE + MAP_LTX,
                      y * UNITSIZE + MAP_LTY,
                      x * UNITSIZE + MAP_LTX + UNITSIZE-3,
                      y * UNITSIZE + MAP_LTY + UNITSIZE-3);
                      
            */
        }
    }
    //    player_x = get_player_x();
    //    player_y = get_player_y();
    //    EzSetColorByName("Yellow");
    //    EzFillCircleB(player_x * UNITSIZE + MAP_LTX,
    //                  player_y * UNITSIZE + MAP_LTY, UNITSIZE/2);
    draw_map_element(15, 1, me_trunk);
    if (get_game_state() == gs_clear){
        SetColor(7);
        EzSetFontSize(32);
        EzDrawStringB(180, 140, "ステージクリア");
        EzSetFontSize(14);
    }
    EzShowBuffer();

}


void draw_route_value(int x, int y){
    char buf[12];
    EzSetColorByRGB(200, 200, 200);
    sprintf(buf, "%3d", get_route_value(x, y));
    EzDrawStringB(x*UNITSIZE + MAP_LTX,
                  y*UNITSIZE + MAP_LTY+14,
                  buf);
}
void draw_route_dir(int x, int y){
    char buf[12];
    EzSetColorByRGB(200, 200, 200);
    sprintf(buf, " %c", Dir2Chr(get_route_dir(x, y)));
    //    sprintf(buf, "%3d", get_route_dir(x, y));
    EzDrawStringB(x*UNITSIZE + MAP_LTX,
                  y*UNITSIZE + MAP_LTY+14,
                  buf);
}

show_route_mode_t get_show_route_mode(void){
    return show_route_mode;
}
void set_show_route_mode(show_route_mode_t mode){
    show_route_mode = mode;
}


/*
void show_route(void){
    is_route_shown = 1;
}
void hide_route(void){
    is_route_shown = 0;
}
void toggle_show_route(void){
    if (is_route_shown){
        hide_route();
    }
    else {
        show_route();
    }
}
*/
