#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <EzGraph.h>
#include "layout.h"
#include "main.h"
#include "game.h"
#include "routing.h"
#include "stage.h"
#include "draw.h"
#include "uty.h"

static int player_x, player_y;
static dir_t input_move_dir;
static int nose_x, nose_y;

static game_state_t game_state;
static int clear_countdown;

game_state_t get_game_state(void){
    return game_state;
}
void set_game_state(game_state_t state){
    game_state = state;
    switch (state){
    case gs_working:
        break;
    case gs_growing:
        nose_x = get_stage_home_x();
        nose_y = get_stage_home_y();
        break;
    case gs_shrinking:
        break;
    case gs_clear:
        clear_countdown = GAME_CLEAR_COUNTDOWN;
        break;
    default:
        break;
    }
}


void game_init(void){
}


void setup_game(void){
    input_move_dir = di_invalid;
    EzSetKeyHandler(game_keyh);
    EzSetKeyReleaseHandler(game_key_releaseh);
    EzSetTimerHandler(game_timerh, ANIME_INTERVAL);
    setup_stage();

    // temp
    player_x = get_stage_player_x();
    player_y = get_stage_player_y();

    setup_routing();
   
    setup_draw_screen();
    draw_screen();
    set_game_state(gs_working);
}


void game_keyh(int key){
    show_route_mode_t show_route_mode;
    //    input_move_dir = di_invalid;
    if (get_game_state() != gs_working) return;  // gs_workingのときのみ操作可能
    switch (key){
    case EZ_Left:
        input_move_dir = di_left;
        break;
    case EZ_Right:
        input_move_dir = di_right;
        break;
    case EZ_Up:
        input_move_dir = di_up;
        break;
    case EZ_Down:
        input_move_dir = di_down;
        break;
    case 'r':
        show_route_mode = get_show_route_mode()+1;
        if (show_route_mode>=sr_end) show_route_mode = sr_begin;
        set_show_route_mode(show_route_mode);
        break;
    default:
        break;
    }
}

void game_key_releaseh(int key){
    switch (key){
    case ' ':
        if (get_game_state() == gs_clear){
            clear_countdown = 0;
        }
        break;
    case 'g':
    case 'q':
        set_main_state(ms_select_stage);        
        break;
    default:
        break;
    }
}

void game_timerh(void){
    dir_t dir;
    switch (game_state){
    case gs_working:
        if (input_move_dir != di_invalid){ // 移動キーが押された
            move(input_move_dir);
            input_move_dir = di_invalid;  // 1入力につき一歩動く
        }
        break;
    case gs_growing:
        if (get_map_element(nose_x, nose_y) != me_food){
            set_map_element(nose_x, nose_y, me_trunk);
            dir = get_route_dir(nose_x, nose_y);
            push_route_stack(nose_x, nose_y);
            nose_x += OffsetX(dir);
            nose_y += OffsetY(dir);
        }
        else {
            delete_stage_food(nose_x, nose_y);
            //            set_map_element(nose_x, nose_y, me_blank);
            set_game_state(gs_shrinking);            
        }
        break;
    case gs_shrinking:
        set_map_element(nose_x, nose_y, me_blank);
        if (pop_route_stack(&nose_x, &nose_y)){
        }
        else {
            if (get_reachable_food_num()>0){
                make_dir_map();
            }
            else {
                if (get_stage_food_num() == 0){
                    set_game_state(gs_clear);
                }
                else {
                    set_game_state(gs_working);
                }
            }
        }
        break;
    case gs_clear:
        clear_countdown--;
        if (clear_countdown < 0){
            set_main_state(ms_select_stage);
        }
    default:
        break;
    }
    draw_screen();
}

void quit_game(void){
}
void retry_game(void){
}

int get_player_x(void){
    return player_x;
}
int get_player_y(void){
    return player_y;
}

void move(dir_t dir){
    int nx, ny, nnx, nny;
    map_element_t me, nme, nnme;

    nx = player_x + OffsetX(dir);
    ny = player_y + OffsetY(dir);
    nnx = nx + OffsetX(dir);
    nny = ny + OffsetY(dir);
    me = get_map_element(player_x, player_y);
    nme = get_map_element(nx, ny);
    nnme = get_map_element(nnx, nny);
    switch(nme){
    case me_blank:
    case me_food:
        //        update_route_value(player_x, player_y);
        //        update_route_value(nx, ny);
        //        expand_route_value();
        move_player(nx, ny, me, nme);
        break;
    case me_wall:
        // 移動できない
        break;
    case me_block:
        if (nnme == me_blank){ // さらに向こうがblankのときのみ押せる
            set_map_element(nx, ny, me_blank);
            set_map_element(nnx, nny, me_block);
            // プレイヤーの移動先はblockが動いた跡のblank
            move_player(nx, ny, me, me_blank);
        }
        break;
    default:
        assert(0);
        break;
    }
    printf("player (%d, %d)\n", player_x, player_y);
}

void move_player(int nx, int ny, map_element_t me, map_element_t nme){
    if (me == me_player){
        set_map_element(player_x, player_y, me_blank);
    }
    else if (me == me_player_food){
        set_map_element(player_x, player_y, me_food);
    }
    else {
        assert(0); // Does not reach here.
    }
    player_x = nx;
    player_y = ny;
    if (nme == me_blank){
        set_map_element(player_x, player_y, me_player);
    }
    else if (nme == me_food){
            set_map_element(player_x, player_y, me_player_food);
    }
    else {
        assert(0); // Does not reach here.
    }
    make_value_map();
    make_dir_map();
}


/*
void clear_food_stack(void){
    food_stack_top = 0;
}
void push_food_stack(int x, int y){
    pos_t pos;
    pos.x = x;
    pos.y = y;
    food_stack[route_stack_top] = pos;
    food_stack_top++;
    assert(food_stack_top < ROUTE_STACK_MAXSIZE);
}

// スタックが空でないなら*x, *yにポップした座標をセットし非0を返す
int pop_food_stack(int *x, int *y){
    if (food_stack_top> 0){
        food_stack_top--;
        *x = food_stack[food_stack_top].x;
        *y = food_stack[food_stack_top].y;
        return 1;
    }
    else {
        return 0;
    }
}

void print_route_stack(void){
    int i;
    printf("ROUTE STACK(%d)[", route_stack_top);
    for (i = 0; i < route_stack_top; i++){
        printf("(%d,%d)", route_stack[i].x, route_stack[i].y);
    }
}
*/

