#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "stage.h"
#include "game.h"
#include "file.h"
#include "jstr.h"

#define BUF_SIZE 1024
static char stage_dpath[BUF_SIZE];
stage_t current_stage;

typedef struct {
    char symbol;
    map_element_t me;
} map_symbol_info_t;

static map_symbol_info_t map_symbol_info_list[] =
    {
     {' ', me_blank},
     {'.', me_blank},
     {'#', me_wall},
     {'$', me_block},
     {'@', me_food},
     {'P', me_player},
     {'H', me_home},
    };

#define MAP_SYMBOL_INFO_NUM \
    (sizeof(map_symbol_info_list)/sizeof(map_symbol_info_t))

map_element_t symbol2me_table[256];
#define MapSymbol2Me(c) (symbol2me_table[(signed char)c])

#define SetThumbColor(x, y, c) (thumb->map[y][x] = c)

void chop_line(char *tline);
void set_stage_home_xy(int x, int y);
void inc_stage_food_num(void);
//void add_stage_food(int x, int y);    
#ifdef DEBUG
void print_stage_info(void);
#endif
    

void stage_init(char *argv0){
    int i;
    map_element_t me;
    char symbol;
    
    printf("stage init %s\n", argv0);
	// Visual Studioでの開発中は、
	// 実行ファイルがソースと異なるディレクトリに作られるので、
	// 実行ファイルからの相対ディレクトリ指定では正しく動作しない
	// カレントディレクトリはソース(プロジェクトファイル?)の場所なので、
	// とりあえずカレントからの相対指定にしておく
#if defined(_MSC_VER) && defined(_DEBUG)
    sprintf(stage_dpath, "%s", STAGE_DIR);
#else
    sprintf(stage_dpath, "%s/%s", get_exe_dpath(argv0), STAGE_DIR);
    printf("dpath:%s\n", stage_dpath);
#endif

    for (i = 0; i < MAP_SYMBOL_INFO_NUM; i++){
        symbol = map_symbol_info_list[i].symbol;
        me = map_symbol_info_list[i].me;
        symbol2me_table[(signed char)symbol] = me;
    }
}


void setup_stage(void){
}

void clear_stage(void){
    int x, y;
    
    set_stage_player_xy(INVALID_X, INVALID_Y);
    set_stage_home_xy(INVALID_X, INVALID_Y);
    current_stage.food_num = 0;
    
    for (y = 0; y < MAP_HEIGHT ; y++){
        for (x = 0; x < MAP_WIDTH; x++){
            set_map_element(x, y, me_blank);
        }
    }
}

void load_stage(char *stage_name){
    FILE *fp;
    char fpath[BUF_SIZE*2];
    char tline[BUF_SIZE];
    static char *sc_str = SC_STR;
    char *scp;
    int line_no = 0;
    int sc, x;
    int map_row = 0;
    map_element_t me;

    clear_stage();
    
    sprintf(fpath, "%s/%s.%s", stage_dpath, stage_name, STAGE_EXT);
    printf("read:%s...", fpath);
    puts("");

        fp = fopen_bomcut(fpath, "r");
    if (fp == NULL){
        log_error("Cannot open stage file %s.\n", fpath);
        exit(1);
    }
    while (fgets(tline, BUF_SIZE, fp) != NULL){
        line_no++;
        printf("line:%d:%s", line_no, tline);
        if (tline[0] == '@'){
            scp = strchr(sc_str, tline[1]);
            if (scp == NULL){
                log_error("Unknown command @%c at line %d in %s\n",
                       tline[1], line_no, stage_name);
                exit(1);
            }
            sc = scp - sc_str+1;
            switch (sc){
            case sc_title:
                break;
            case sc_map_begin:
                break;
            case sc_map_end:
                // マップ内にプレイヤーが指定されていなければエラー
                if (get_stage_player_x() == INVALID_X){
                    log_error("Player not specified in %s\n", stage_name);
                    exit(1);
                }
                // マップ内にホームが指定されていなければエラー
                if (get_stage_home_x() == INVALID_X){
                    log_error("Home not specified in %s\n", stage_name);
                    exit(1);
                }
                break;
            default:
                log_error("Unknown command at line %d in %s\n",
                          line_no, stage_name);
                exit(1);
            }
        }
        else { // コマンド行以外
            // 改行コードを削除
            chop_line(tline);
            switch (sc){
            case sc_map_begin:
                if (map_row >= MAP_HEIGHT){
                    log_error("Too many lines in map at line %d in %s\n",
                              line_no, stage_name);
                    exit(1);
                }
                // !!横幅エラーチェックを作る
                x = 0;
                while (tline[x] != '\0'){
                    me = MapSymbol2Me(tline[x]);
                    //                    printf("%c(%2x) ", tline[x], me);
                    if (x >= MAP_WIDTH+1){
                        log_error("Length of line %d exceeds MAP_WIDTH in %s\n",
                                  line_no, stage_name);
                        exit(1);
                    }
                    switch (me){ // マップ要素によっては個別処理を行う
                    case me_player:
                        set_stage_player_xy(x, map_row);
                        break;
                    case me_food:
                        inc_stage_food_num();
                        break;
                    case me_home:
                        set_stage_home_xy(x, map_row);
                        me = me_blank; // me_homeは以後blank扱い
                        break;
                    default:
                        break;
                    }
                    set_map_element(x, map_row, me);
                    x++;
                }
                //                printf("\n");
                    map_row ++;
                break;
            }
        }
    }
#ifdef DEBUG
    print_stage_info();
#endif
}

void get_thumb(char *stage_name, stage_thumb_t *thumb, int stage_no){
    int x, y;
    map_element_t me;

    load_stage(stage_name);

	for (y = 0 ; y < MAP_HEIGHT ; y++){
        for (x = 0 ; x < MAP_WIDTH ; x++){
            me = get_map_element(x, y);
            switch(me){
            case me_wall:
                SetThumbColor(x, y, 2);
				break;
            case me_block:
                SetThumbColor(x, y, 1);
				break;
            case me_player:
                SetThumbColor(x, y, 7);
				break;
            case me_food:
                SetThumbColor(x, y, 6);
				break;
            default:
                break;
            }
        }
    }
}

// 行末の改行を取り除く
void chop_line(char *tline){
    int last = strlen(tline)-1;
    if (tline[last] == '\n'){
        tline[last] = '\0';
    }
}


map_element_t get_map_element(int x, int y){
    return current_stage.map[y][x];
}

void set_map_element(int x, int y, map_element_t me){   
    current_stage.map[y][x]=me;
}

void set_stage_player_xy(int x, int y){
    current_stage.player_x = x;
    current_stage.player_y = y;
}
int get_stage_player_x(void){
    return current_stage.player_x;
}
int get_stage_player_y(void){
    return current_stage.player_y;
}

void set_stage_home_xy(int x, int y){
    current_stage.home_x = x;
    current_stage.home_y = y;
}
int get_stage_home_x(void){
    return current_stage.home_x;
}
int get_stage_home_y(void){
    return current_stage.home_y;
}

int get_stage_food_num(void){
    return current_stage.food_num;
}
void inc_stage_food_num(void){
    current_stage.food_num++;
}
void delete_stage_food(int x, int y){
    set_map_element(x, y, me_blank);
    current_stage.food_num--;
    printf("food left:%d\n", current_stage.food_num);
}




#ifdef DEBUG
void print_stage_info(void){
    printf("player (%d, %d)\n", get_stage_player_x(), get_stage_player_y());
    printf("home (%d, %d)\n", get_stage_home_x(), get_stage_home_y());
    printf("food num:%d\n", get_stage_food_num());
}
#endif
        
/*
int get_stage_food_x(int id){
    assert(0<=id && id < current_stage.food_num);
    return current_stage.food_x[id];
}
int get_stage_food_y(int id){
    assert(0<=id && id < current_stage.food_num);
    return current_stage.food_y[id];
}
void add_stage_food(int x, int y){
    curren_stage.food_x[current_stage.food_num] = x;
    curren_stage.food_y[current_stage.food_num] = y;
    current_stage.food_num++;
}
*/    

//void delete_stage_food(int id){
//}

